@@ -15,19 +15,19 @@
  * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __CLUTTER_STAGE_VIEW_H__
-#define __CLUTTER_STAGE_VIEW_H__
+#pragma once
 
 #if !defined(__CLUTTER_H_INSIDE__) && !defined(CLUTTER_COMPILATION)
 #error "Only <clutter/clutter.h> can be included directly."
 #endif
 
-#include <cairo.h>
 #include <glib-object.h>
-#include <cogl/cogl.h>
 
-#include "clutter-macros.h"
-#include "clutter-frame-clock.h"
+#include "cogl/cogl.h"
+#include "clutter/clutter-macros.h"
+#include "clutter/clutter-frame-clock.h"
+#include "clutter/clutter-types.h"
+#include "mtk/mtk.h"
 
 #define CLUTTER_TYPE_STAGE_VIEW (clutter_stage_view_get_type ())
 CLUTTER_EXPORT
@@ -39,32 +39,24 @@ struct _ClutterStageViewClass
 {
   GObjectClass parent_class;
 
-  void (* setup_offscreen_blit_pipeline) (ClutterStageView *view,
-                                          CoglPipeline     *pipeline);
+  ClutterFrame * (* new_frame) (ClutterStageView *view);
 
-  void (* get_offscreen_transformation_matrix) (ClutterStageView  *view,
-                                                graphene_matrix_t *matrix);
+  ClutterPaintFlag (* get_default_paint_flags) (ClutterStageView *view);
 
-  void (* transform_rect_to_onscreen) (ClutterStageView            *view,
-                                       const cairo_rectangle_int_t *src_rect,
-                                       int                          dst_width,
-                                       int                          dst_height,
-                                       cairo_rectangle_int_t       *dst_rect);
+  void (* schedule_update) (ClutterStageView *view);
 };
 
 CLUTTER_EXPORT
 void clutter_stage_view_destroy (ClutterStageView *view);
 
 CLUTTER_EXPORT
-void clutter_stage_view_get_layout (ClutterStageView      *view,
-                                    cairo_rectangle_int_t *rect);
+void clutter_stage_view_get_layout (ClutterStageView *view,
+                                    MtkRectangle     *rect);
 
 CLUTTER_EXPORT
 CoglFramebuffer *clutter_stage_view_get_framebuffer (ClutterStageView *view);
 CLUTTER_EXPORT
 CoglFramebuffer *clutter_stage_view_get_onscreen (ClutterStageView *view);
-CLUTTER_EXPORT
-void             clutter_stage_view_invalidate_offscreen_blit_pipeline (ClutterStageView *view);
 
 CLUTTER_EXPORT
 float clutter_stage_view_get_scale (ClutterStageView *view);
@@ -82,4 +74,20 @@ CoglScanout * clutter_stage_view_peek_scanout (ClutterStageView *view);
 CLUTTER_EXPORT
 float clutter_stage_view_get_refresh_rate (ClutterStageView *view);
 
-#endif /* __CLUTTER_STAGE_VIEW_H__ */
+CLUTTER_EXPORT
+gboolean clutter_stage_view_has_shadowfb (ClutterStageView *view);
+
+CLUTTER_EXPORT
+void clutter_stage_view_schedule_update_now (ClutterStageView *view);
+
+CLUTTER_EXPORT
+ClutterPaintFlag clutter_stage_view_get_default_paint_flags (ClutterStageView *view);
+
+CLUTTER_EXPORT
+ClutterColorState * clutter_stage_view_get_color_state (ClutterStageView *view);
+
+CLUTTER_EXPORT
+ClutterColorState * clutter_stage_view_get_output_color_state (ClutterStageView *view);
+
+CLUTTER_EXPORT
+MtkMonitorTransform clutter_stage_view_get_transform (ClutterStageView *view);
