@@ -42,10 +42,12 @@
 #include "cogl/cogl-framebuffer-private.h"
 #include "cogl/cogl-blit.h"
 #include "cogl/cogl-private.h"
+#include "cogl/driver/gl/cogl-driver-gl-private.h"
+#include "cogl/driver/gl/cogl-texture-driver-gl-private.h"
 
 #include <stdlib.h>
 
-G_DEFINE_TYPE (CoglAtlas, cogl_atlas, G_TYPE_OBJECT);
+G_DEFINE_FINAL_TYPE (CoglAtlas, cogl_atlas, G_TYPE_OBJECT);
 
 static void
 cogl_atlas_dispose (GObject *object)
@@ -193,6 +195,12 @@ _cogl_atlas_get_initial_size (CoglContext *ctx,
                               unsigned int *map_width,
                               unsigned int *map_height)
 {
+  CoglDriverGL *driver_gl = COGL_DRIVER_GL (ctx->driver);
+  CoglDriverGLClass *driver_klass = COGL_DRIVER_GL_GET_CLASS (driver_gl);
+  CoglTextureDriverGL *tex_driver_gl =
+    COGL_TEXTURE_DRIVER_GL (ctx->texture_driver);
+  CoglTextureDriverGLClass *tex_driver_klass =
+    COGL_TEXTURE_DRIVER_GL_GET_CLASS (tex_driver_gl);
   unsigned int size;
   GLenum gl_intformat;
   GLenum gl_format;
@@ -200,11 +208,12 @@ _cogl_atlas_get_initial_size (CoglContext *ctx,
 
   g_return_if_fail (cogl_pixel_format_get_n_planes (format) == 1);
 
-  ctx->driver_vtable->pixel_format_to_gl (ctx,
-                                          format,
-                                          &gl_intformat,
-                                          &gl_format,
-                                          &gl_type);
+  driver_klass->pixel_format_to_gl (driver_gl,
+                                    ctx,
+                                    format,
+                                    &gl_intformat,
+                                    &gl_format,
+                                    &gl_type);
 
   /* At least on Intel hardware, the texture size will be rounded up
      to at least 1MB so we might as well try to aim for that as an
@@ -219,12 +228,13 @@ _cogl_atlas_get_initial_size (CoglContext *ctx,
   /* Some platforms might not support this large size so we'll
      decrease the size until it can */
   while (size > 1 &&
-         !ctx->texture_driver->size_supported (ctx,
-                                               GL_TEXTURE_2D,
-                                               gl_intformat,
-                                               gl_format,
-                                               gl_type,
-                                               size, size))
+         !tex_driver_klass->size_supported (tex_driver_gl,
+                                            ctx,
+                                            GL_TEXTURE_2D,
+                                            gl_intformat,
+                                            gl_format,
+                                            gl_type,
+                                            size, size))
     size >>= 1;
 
   *map_width = size;
@@ -239,24 +249,32 @@ _cogl_atlas_create_map (CoglContext             *ctx,
                         unsigned int             n_textures,
                         CoglAtlasRepositionData *textures)
 {
+  CoglDriverGL *driver_gl = COGL_DRIVER_GL (ctx->driver);
+  CoglDriverGLClass *driver_klass = COGL_DRIVER_GL_GET_CLASS (driver_gl);
+  CoglTextureDriverGL *tex_driver_gl =
+    COGL_TEXTURE_DRIVER_GL (ctx->texture_driver);
+  CoglTextureDriverGLClass *tex_driver_klass =
+    COGL_TEXTURE_DRIVER_GL_GET_CLASS (tex_driver_gl);
   GLenum gl_intformat;
   GLenum gl_format;
   GLenum gl_type;
 
-  ctx->driver_vtable->pixel_format_to_gl (ctx,
-                                          format,
-                                          &gl_intformat,
-                                          &gl_format,
-                                          &gl_type);
+  driver_klass->pixel_format_to_gl (driver_gl,
+                                    ctx,
+                                    format,
+                                    &gl_intformat,
+                                    &gl_format,
+                                    &gl_type);
 
   /* Keep trying increasingly larger atlases until we can fit all of
      the textures */
-  while (ctx->texture_driver->size_supported (ctx,
-                                              GL_TEXTURE_2D,
-                                              gl_intformat,
-                                              gl_format,
-                                              gl_type,
-                                              map_width, map_height))
+  while (tex_driver_klass->size_supported (tex_driver_gl,
+                                           ctx,
+                                           GL_TEXTURE_2D,
+                                           gl_intformat,
+                                           gl_format,
+                                           gl_type,
+                                           map_width, map_height))
     {
       CoglRectangleMap *new_atlas = _cogl_rectangle_map_new (map_width,
                                                              map_height,
