@@ -44,7 +44,6 @@ typedef enum
   COGL_DEBUG_FRAMEBUFFER,
   COGL_DEBUG_OFFSCREEN,
   COGL_DEBUG_DRAW,
-  COGL_DEBUG_PANGO,
   COGL_DEBUG_RECTANGLES,
   COGL_DEBUG_OBJECT,
   COGL_DEBUG_BLEND_STRINGS,
@@ -60,6 +59,7 @@ typedef enum
   COGL_DEBUG_OPENGL,
   COGL_DEBUG_DISABLE_TEXTURING,
   COGL_DEBUG_SHOW_SOURCE,
+  COGL_DEBUG_SHOW_UNIFORMS,
   COGL_DEBUG_DISABLE_BLENDING,
   COGL_DEBUG_TEXTURE_PIXMAP,
   COGL_DEBUG_BITMAP,
