@@ -32,15 +32,15 @@
 
 #include "cogl/cogl-display.h"
 #include "cogl/cogl-renderer.h"
-#include "cogl/cogl-onscreen-template-private.h"
 
 struct _CoglDisplay
 {
   GObjectClass parnet_class;
 
+  CoglContext *context;
+
   gboolean setup;
   CoglRenderer *renderer;
-  CoglOnscreenTemplate *onscreen_template;
 
   void *winsys;
 };
