@@ -47,7 +47,6 @@
 #include "cogl/cogl-context-private.h"
 #include "cogl/cogl-spans.h"
 #include "cogl/cogl-journal-private.h"
-#include "cogl/cogl-primitive-texture.h"
 #include "cogl/driver/gl/cogl-texture-gl-private.h"
 
 #include <string.h>
@@ -58,7 +57,7 @@ G_DEFINE_FINAL_TYPE (CoglTexture2DSliced, cogl_texture_2d_sliced, COGL_TYPE_TEXT
 
 typedef struct _ForeachData
 {
-  CoglMetaTextureCallback callback;
+  CoglTextureForeachCallback callback;
   void *user_data;
   float x_normalize_factor;
   float y_normalize_factor;
@@ -132,8 +131,8 @@ _cogl_rect_slices_for_size (int     size_to_fill,
       /* Add another slice span of same size */
       if (out_spans)
         g_array_append_val (out_spans, span);
-      span.start   += span.size;
-      size_to_fill -= span.size;
+      span.start   += (int) span.size;
+      size_to_fill -= (int) span.size;
       n_spans++;
     }
 
@@ -158,6 +157,8 @@ setup_spans (CoglContext *ctx,
              CoglPixelFormat internal_format,
              GError **error)
 {
+  CoglTextureDriverClass *tex_driver =
+    COGL_TEXTURE_DRIVER_GET_CLASS (ctx->texture_driver);
   int max_width;
   int max_height;
   int n_x_slices;
@@ -176,10 +177,11 @@ setup_spans (CoglContext *ctx,
       CoglSpan span;
 
       /* Check if size supported else bail out */
-      if (!ctx->driver_vtable->texture_2d_can_create (ctx,
-                                                      max_width,
-                                                      max_height,
-                                                      internal_format))
+      if (!tex_driver->texture_2d_can_create (ctx->texture_driver,
+                                              ctx,
+                                              max_width,
+                                              max_height,
+                                              internal_format))
         {
           g_set_error (error, COGL_TEXTURE_ERROR, COGL_TEXTURE_ERROR_SIZE,
                        "Sliced texture size of %d x %d not possible "
@@ -214,10 +216,11 @@ setup_spans (CoglContext *ctx,
   else
     {
       /* Decrease the size of largest slice until supported by GL */
-      while (!ctx->driver_vtable->texture_2d_can_create (ctx,
-                                                         max_width,
-                                                         max_height,
-                                                         internal_format))
+      while (!tex_driver->texture_2d_can_create (ctx->texture_driver,
+                                                 ctx,
+                                                 max_width,
+                                                 max_height,
+                                                 internal_format))
         {
           /* Alternate between width and height */
           if (max_width > max_height)
@@ -322,7 +325,8 @@ allocate_slices (CoglTexture2DSliced *tex_2ds,
 
           slice =
             cogl_texture_2d_new_with_size (ctx,
-                                           x_span->size, y_span->size);
+                                           (int) x_span->size,
+                                           (int) y_span->size);
 
           _cogl_texture_copy_internal_format (tex, slice);
 
@@ -392,8 +396,8 @@ _cogl_texture_2d_sliced_allocate_waste_buffer (CoglTexture2DSliced *tex_2ds,
         = &g_array_index (tex_2ds->slice_x_spans, CoglSpan, 0);
       CoglSpan  *first_y_span
         = &g_array_index (tex_2ds->slice_y_spans, CoglSpan, 0);
-      unsigned int right_size = first_y_span->size * last_x_span->waste;
-      unsigned int bottom_size = first_x_span->size * last_y_span->waste;
+      unsigned int right_size = (unsigned int) (first_y_span->size * last_x_span->waste);
+      unsigned int bottom_size = (unsigned int) (first_x_span->size * last_y_span->waste);
 
       waste_buf = g_malloc (MAX (right_size, bottom_size) * bpp);
     }
@@ -472,25 +476,25 @@ _cogl_texture_2d_sliced_set_waste (CoglTexture2DSliced *tex_2ds,
             }
 
           waste_bmp = cogl_bitmap_new_for_data (ctx,
-                                                x_span->waste,
-                                                y_iter->intersect_end -
-                                                y_iter->intersect_start,
+                                                (int) x_span->waste,
+                                                (int) (y_iter->intersect_end -
+                                                       y_iter->intersect_start),
                                                 source_format,
-                                                x_span->waste * bpp,
+                                                (int) (x_span->waste * bpp),
                                                 waste_buf);
 
           if (!_cogl_texture_set_region_from_bitmap (COGL_TEXTURE (slice_tex),
                                                      0, /* src_x */
                                                      0, /* src_y */
-                                                     x_span->waste, /* width */
+                                                     (int) x_span->waste, /* width */
                                                      /* height */
-                                                     y_iter->intersect_end -
-                                                     y_iter->intersect_start,
+                                                     (int) (y_iter->intersect_end -
+                                                            y_iter->intersect_start),
                                                      waste_bmp,
                                                      /* dst_x */
-                                                     x_span->size - x_span->waste,
-                                                     y_iter->intersect_start -
-                                                     y_span->start, /* dst_y */
+                                                     (int) (x_span->size - x_span->waste),
+                                                     (int) (y_iter->intersect_start -
+                                                            y_span->start), /* dst_y */
                                                      0, /* level */
                                                      error))
             {
@@ -515,11 +519,11 @@ _cogl_texture_2d_sliced_set_waste (CoglTexture2DSliced *tex_2ds,
 
           if (x_iter->intersect_end - x_iter->pos
               >= x_span->size - x_span->waste)
-            copy_width = x_span->size + x_iter->pos - x_iter->intersect_start;
+            copy_width = (unsigned int) (x_span->size + x_iter->pos - x_iter->intersect_start);
           else
-            copy_width = x_iter->intersect_end - x_iter->intersect_start;
+            copy_width = (unsigned int) (x_iter->intersect_end - x_iter->intersect_start);
 
-          intersect_width = x_iter->intersect_end - x_iter->intersect_start;
+          intersect_width = (unsigned int) (x_iter->intersect_end - x_iter->intersect_start);
 
           for (wy = 0; wy < y_span->waste; wy++)
             {
@@ -535,7 +539,7 @@ _cogl_texture_2d_sliced_set_waste (CoglTexture2DSliced *tex_2ds,
 
           waste_bmp = cogl_bitmap_new_for_data (ctx,
                                                 copy_width,
-                                                y_span->waste,
+                                                (int) y_span->waste,
                                                 source_format,
                                                 copy_width * bpp,
                                                 waste_buf);
@@ -544,13 +548,13 @@ _cogl_texture_2d_sliced_set_waste (CoglTexture2DSliced *tex_2ds,
                                                      0, /* src_x */
                                                      0, /* src_y */
                                                      copy_width, /* width */
-                                                     y_span->waste, /* height */
+                                                     (int) y_span->waste, /* height */
                                                      waste_bmp,
                                                      /* dst_x */
-                                                     x_iter->intersect_start -
-                                                     x_iter->pos,
+                                                     (int) (x_iter->intersect_start -
+                                                            x_iter->pos),
                                                      /* dst_y */
-                                                     y_span->size - y_span->waste,
+                                                     (int) (y_span->size - y_span->waste),
                                                      0, /* level */
                                                      error))
             {
@@ -603,12 +607,12 @@ _cogl_texture_2d_sliced_upload_bitmap (CoglTexture2DSliced *tex_2ds,
                                      CoglTexture2D *, slice_num);
 
           if (!_cogl_texture_set_region_from_bitmap (COGL_TEXTURE (slice_tex),
-                                                     x_span->start, /* src x */
-                                                     y_span->start, /* src y */
-                                                     x_span->size -
-                                                     x_span->waste, /* width */
-                                                     y_span->size -
-                                                     y_span->waste, /* height */
+                                                     (int) x_span->start, /* src x */
+                                                     (int) y_span->start, /* src y */
+                                                     (int) (x_span->size -
+                                                            x_span->waste), /* width */
+                                                     (int) (y_span->size -
+                                                            y_span->waste), /* height */
                                                      bmp,
                                                      0, /* dst x */
                                                      0, /* dst y */
@@ -731,14 +735,6 @@ _cogl_texture_2d_sliced_allocate (CoglTexture *tex,
   g_return_val_if_reached (FALSE);
 }
 
-static int
-_cogl_texture_2d_sliced_get_max_waste (CoglTexture *tex)
-{
-  CoglTexture2DSliced *tex_2ds = COGL_TEXTURE_2D_SLICED (tex);
-
-  return tex_2ds->max_waste;
-}
-
 static gboolean
 _cogl_texture_2d_sliced_is_sliced (CoglTexture *tex)
 {
@@ -746,7 +742,7 @@ _cogl_texture_2d_sliced_is_sliced (CoglTexture *tex)
 
   /* It's only after allocating a sliced texture that we will know
    * whether it really needed to be sliced... */
-  if (!tex->allocated)
+  if (!cogl_texture_is_allocated (tex))
     cogl_texture_allocate (tex, NULL);
 
   if (tex_2ds->slice_x_spans->len != 1 ||
@@ -800,7 +796,8 @@ _cogl_texture_2d_sliced_transform_coords_to_gl (CoglTexture *tex,
 
   /* Let the child texture further transform the coords */
   slice_tex = g_array_index (tex_2ds->slice_textures, CoglTexture2D *, 0);
-  _cogl_texture_transform_coords_to_gl (COGL_TEXTURE (slice_tex), s, t);
+
+  COGL_TEXTURE_GET_CLASS (slice_tex)->transform_coords_to_gl (COGL_TEXTURE (slice_tex), s, t);
 }
 
 static CoglTransformResult
@@ -905,7 +902,7 @@ _cogl_texture_2d_sliced_ensure_non_quad_rendering (CoglTexture *tex)
     {
       CoglTexture2D *slice_tex = g_array_index (tex_2ds->slice_textures,
                                                 CoglTexture2D *, i);
-      _cogl_texture_ensure_non_quad_rendering (COGL_TEXTURE (slice_tex));
+      COGL_TEXTURE_GET_CLASS (slice_tex)->ensure_non_quad_rendering (COGL_TEXTURE (slice_tex));
     }
 }
 
@@ -1011,12 +1008,12 @@ _cogl_texture_2d_sliced_upload_subregion (CoglTexture2DSliced *tex_2ds,
                                    x_iter.index);
 
           /* Pick intersection width and height */
-          inter_w =  (x_iter.intersect_end - x_iter.intersect_start);
-          inter_h =  (y_iter.intersect_end - y_iter.intersect_start);
+          inter_w = (int) (x_iter.intersect_end - x_iter.intersect_start);
+          inter_h = (int) (y_iter.intersect_end - y_iter.intersect_start);
 
           /* Localize intersection top-left corner to slice*/
-          local_x =  (x_iter.intersect_start - x_iter.pos);
-          local_y =  (y_iter.intersect_start - y_iter.pos);
+          local_x = (int) (x_iter.intersect_start - x_iter.pos);
+          local_y = (int) (y_iter.intersect_start - y_iter.pos);
 
           slice_num = y_iter.index * tex_2ds->slice_x_spans->len + x_iter.index;
 
@@ -1080,7 +1077,7 @@ _cogl_texture_2d_sliced_set_region (CoglTexture *tex,
   gboolean status;
 
   upload_bmp = _cogl_bitmap_convert_for_upload (bmp,
-                                                _cogl_texture_get_format (tex),
+                                                cogl_texture_get_format (tex),
                                                 error);
   if (!upload_bmp)
     return FALSE;
@@ -1120,13 +1117,13 @@ re_normalize_sub_texture_coords_cb (CoglTexture *sub_texture,
 
 static void
 _cogl_texture_2d_sliced_foreach_sub_texture_in_region (
-                                       CoglTexture *tex,
-                                       float virtual_tx_1,
-                                       float virtual_ty_1,
-                                       float virtual_tx_2,
-                                       float virtual_ty_2,
-                                       CoglMetaTextureCallback callback,
-                                       void *user_data)
+                                       CoglTexture                *tex,
+                                       float                       virtual_tx_1,
+                                       float                       virtual_ty_1,
+                                       float                       virtual_tx_2,
+                                       float                       virtual_ty_2,
+                                       CoglTextureForeachCallback  callback,
+                                       void                       *user_data)
 {
   CoglTexture2DSliced *tex_2ds = COGL_TEXTURE_2D_SLICED (tex);
   CoglSpan *x_spans = (CoglSpan *)tex_2ds->slice_x_spans->data;
@@ -1187,7 +1184,6 @@ cogl_texture_2d_sliced_class_init (CoglTexture2DSlicedClass *klass)
   texture_class->allocate = _cogl_texture_2d_sliced_allocate;
   texture_class->set_region = _cogl_texture_2d_sliced_set_region;
   texture_class->foreach_sub_texture_in_region = _cogl_texture_2d_sliced_foreach_sub_texture_in_region;
-  texture_class->get_max_waste = _cogl_texture_2d_sliced_get_max_waste;
   texture_class->is_sliced = _cogl_texture_2d_sliced_is_sliced;
   texture_class->can_hardware_repeat = _cogl_texture_2d_sliced_can_hardware_repeat;
   texture_class->transform_coords_to_gl = _cogl_texture_2d_sliced_transform_coords_to_gl;
@@ -1204,9 +1200,6 @@ cogl_texture_2d_sliced_class_init (CoglTexture2DSlicedClass *klass)
 static void
 cogl_texture_2d_sliced_init (CoglTexture2DSliced *self)
 {
-  CoglTexture *texture = COGL_TEXTURE (self);
-
-  texture->is_primitive = FALSE;
 }
 
 static CoglTexture *
@@ -1236,8 +1229,8 @@ cogl_texture_2d_sliced_new_with_size (CoglContext *ctx,
                                       int height,
                                       int max_waste)
 {
-  CoglTextureLoader *loader = _cogl_texture_create_loader ();
-  loader->src_type = COGL_TEXTURE_SOURCE_TYPE_SIZE;
+  CoglTextureLoader *loader =
+    cogl_texture_loader_new (COGL_TEXTURE_SOURCE_TYPE_SIZE);
   loader->src.sized.width = width;
   loader->src.sized.height = height;
   loader->src.sized.format = COGL_PIXEL_FORMAT_ANY;
@@ -1258,8 +1251,7 @@ cogl_texture_2d_sliced_new_from_bitmap (CoglBitmap *bmp,
 
   g_return_val_if_fail (COGL_IS_BITMAP (bmp), NULL);
 
-  loader = _cogl_texture_create_loader ();
-  loader->src_type = COGL_TEXTURE_SOURCE_TYPE_BITMAP;
+  loader = cogl_texture_loader_new (COGL_TEXTURE_SOURCE_TYPE_BITMAP);
   loader->src.bitmap.bitmap = g_object_ref (bmp);
 
   return _cogl_texture_2d_sliced_create_base (_cogl_bitmap_get_context (bmp),
@@ -1269,46 +1261,3 @@ cogl_texture_2d_sliced_new_from_bitmap (CoglBitmap *bmp,
                                               cogl_bitmap_get_format (bmp),
                                               loader);
 }
-
-CoglTexture *
-cogl_texture_2d_sliced_new_from_data (CoglContext *ctx,
-                                      int width,
-                                      int height,
-                                      int max_waste,
-                                      CoglPixelFormat format,
-                                      int rowstride,
-                                      const uint8_t *data,
-                                      GError **error)
-{
-  CoglBitmap *bmp;
-  CoglTexture *tex_2ds;
-
-  g_return_val_if_fail (format != COGL_PIXEL_FORMAT_ANY, NULL);
-  g_return_val_if_fail (cogl_pixel_format_get_n_planes (format) == 1, NULL);
-  g_return_val_if_fail (data != NULL, NULL);
-
-  /* Rowstride from width if not given */
-  if (rowstride == 0)
-    rowstride = width * cogl_pixel_format_get_bytes_per_pixel (format, 0);
-
-  /* Wrap the data into a bitmap */
-  bmp = cogl_bitmap_new_for_data (ctx,
-                                  width, height,
-                                  format,
-                                  rowstride,
-                                  (uint8_t *) data);
-
-  tex_2ds = cogl_texture_2d_sliced_new_from_bitmap (bmp, max_waste);
-
-  g_object_unref (bmp);
-
-  if (tex_2ds &&
-      !cogl_texture_allocate (COGL_TEXTURE (tex_2ds), error))
-    {
-      g_object_unref (tex_2ds);
-      return NULL;
-    }
-
-  return tex_2ds;
-}
-
