@@ -39,7 +39,7 @@
 #include "cogl/cogl-feature-private.h"
 #include "cogl/cogl-pipeline-private.h"
 #include "cogl/driver/gl/cogl-util-gl-private.h"
-#include "cogl/driver/gl/cogl-pipeline-opengl-private.h"
+#include "cogl/driver/gl/cogl-pipeline-gl-private.h"
 
 #include "cogl/cogl-context-private.h"
 #include "cogl/cogl-pipeline-state-private.h"
