@@ -282,7 +282,8 @@ query_file_info_cb (GObject      *source_object,
   info = g_file_query_info_finish (file, res, &error);
   if (!info)
     {
-      if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
+      if (g_error_matches (error, G_IO_ERROR, G_IO_ERROR_CANCELLED) ||
+          g_error_matches (error, G_IO_ERROR, G_IO_ERROR_NOT_FOUND))
         return;
 
       g_warning ("Failed to query file info on '%s': %s",
