@@ -14,16 +14,13 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  * Written by:
  *     Jasper St. Pierre <jstpierre@mecheye.net>
  */
 
-#ifndef META_CURSOR_RENDERER_H
-#define META_CURSOR_RENDERER_H
+#pragma once
 
 #include <glib-object.h>
 
@@ -73,12 +70,11 @@ graphene_rect_t meta_cursor_renderer_calculate_rect (MetaCursorRenderer *rendere
 
 void meta_cursor_renderer_emit_painted (MetaCursorRenderer *renderer,
                                         MetaCursorSprite   *cursor_sprite,
-                                        ClutterStageView   *stage_view);
+                                        ClutterStageView   *stage_view,
+                                        int64_t             view_frame_counter);
 ClutterInputDevice * meta_cursor_renderer_get_input_device (MetaCursorRenderer *renderer);
 
 void meta_cursor_renderer_update_stage_overlay (MetaCursorRenderer *renderer,
                                                 MetaCursorSprite   *cursor_sprite);
 
 MetaBackend * meta_cursor_renderer_get_backend (MetaCursorRenderer *renderer);
-
-#endif /* META_CURSOR_RENDERER_H */
