@@ -34,6 +34,7 @@ typedef struct _MetaMonitorConfig
   gboolean has_max_bpc;
   unsigned int max_bpc;
   MetaOutputRGBRange rgb_range;
+  MetaColorMode color_mode;
 } MetaMonitorConfig;
 
 typedef struct _MetaLogicalMonitorConfig
@@ -67,6 +68,7 @@ struct _MetaMonitorsConfig
   GList *logical_monitor_configs;
 
   GList *disabled_monitor_specs;
+  GList *for_lease_monitor_specs;
 
   MetaMonitorsConfigFlag flags;
 
@@ -142,6 +144,7 @@ void meta_monitor_config_manager_save_current (MetaMonitorConfigManager *config_
 META_EXPORT_TEST
 MetaMonitorsConfig * meta_monitors_config_new_full (GList                        *logical_monitor_configs,
                                                     GList                        *disabled_monitors,
+                                                    GList                        *for_lease_monitors,
                                                     MetaLogicalMonitorLayoutMode  layout_mode,
                                                     MetaMonitorsConfigFlag        flags);
 
@@ -185,6 +188,10 @@ META_EXPORT_TEST
 gboolean meta_logical_monitor_configs_have_monitor (GList           *logical_monitor_configs,
                                                     MetaMonitorSpec *monitor_spec);
 
+gboolean meta_logical_monitor_configs_have_visible_monitor (MetaMonitorManager *monitor_manager,
+                                                            GList              *logical_monitor_configs,
+                                                            MetaMonitor        *monitor);
+
 META_EXPORT_TEST
 gboolean meta_verify_monitor_mode_spec (MetaMonitorModeSpec *monitor_mode_spec,
                                         GError             **error);
