@@ -89,6 +89,14 @@
  *         <serial>Serial C</serial>
  *       </monitorspec>
  *     </disabled>
+ *     <forlease>
+ *       <monitorspec>
+ *         <connector>LVDS3</connector>
+ *         <vendor>Vendor C</vendor>
+ *         <product>Product C</product>
+ *         <serial>Serial C</serial>
+ *       </monitorspec>
+ *     </forlease>
  *   </configuration>
  * </monitors>
  *
@@ -157,7 +165,9 @@ typedef enum
   STATE_MONITOR_UNDERSCANNING,
   STATE_MONITOR_MAXBPC,
   STATE_MONITOR_RGB_RANGE,
+  STATE_MONITOR_COLOR_MODE,
   STATE_DISABLED,
+  STATE_FOR_LEASE,
   STATE_POLICY,
   STATE_STORES,
   STATE_STORE,
@@ -184,6 +194,7 @@ typedef struct
   MetaMonitorConfig *current_monitor_config;
   MetaLogicalMonitorConfig *current_logical_monitor_config;
   GList *current_disabled_monitor_specs;
+  GList *current_for_lease_monitor_specs;
   gboolean seen_policy;
   gboolean seen_stores;
   gboolean seen_dbus;
@@ -333,6 +344,10 @@ handle_start_element (GMarkupParseContext  *context,
           {
             parser->state = STATE_DISABLED;
           }
+        else if (g_str_equal (element_name, "forlease"))
+          {
+            parser->state = STATE_FOR_LEASE;
+          }
         else
           {
             enter_unknown_element (parser, element_name,
@@ -450,6 +465,10 @@ handle_start_element (GMarkupParseContext  *context,
           {
             parser->state = STATE_MONITOR_RGB_RANGE;
           }
+        else if (g_str_equal (element_name, "colormode"))
+          {
+            parser->state = STATE_MONITOR_COLOR_MODE;
+          }
         else
           {
             g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_UNKNOWN_ELEMENT,
@@ -562,6 +581,13 @@ handle_start_element (GMarkupParseContext  *context,
         return;
       }
 
+    case STATE_MONITOR_COLOR_MODE:
+      {
+        g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_UNKNOWN_ELEMENT,
+                     "Invalid element '%s' under colormode", element_name);
+        return;
+      }
+
     case STATE_DISABLED:
       {
         if (!g_str_equal (element_name, "monitorspec"))
@@ -578,6 +604,22 @@ handle_start_element (GMarkupParseContext  *context,
         return;
       }
 
+    case STATE_FOR_LEASE:
+      {
+        if (!g_str_equal (element_name, "monitorspec"))
+          {
+            g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_UNKNOWN_ELEMENT,
+                         "Invalid element '%s' under forlease", element_name);
+            return;
+          }
+
+        parser->current_monitor_spec = g_new0 (MetaMonitorSpec, 1);
+        parser->monitor_spec_parent_state = STATE_FOR_LEASE;
+        parser->state = STATE_MONITOR_SPEC;
+
+        return;
+      }
+
     case STATE_POLICY:
       {
         if (!(parser->extra_config_flags &
@@ -673,6 +715,15 @@ finish_monitor_spec (ConfigParser *parser)
                           parser->current_monitor_spec);
         parser->current_monitor_spec = NULL;
 
+        return;
+      }
+    case STATE_FOR_LEASE:
+      {
+        parser->current_for_lease_monitor_specs =
+          g_list_prepend (parser->current_for_lease_monitor_specs,
+                          parser->current_monitor_spec);
+        parser->current_monitor_spec = NULL;
+
         return;
       }
 
@@ -729,12 +780,15 @@ static gboolean
 detect_layout_mode_configs (MetaMonitorManager      *monitor_manager,
                             GList                   *logical_monitor_configs,
                             GList                   *disabled_monitor_specs,
+                            GList                   *for_lease_monitor_specs,
                             MetaMonitorsConfigFlag   config_flags,
                             MetaMonitorsConfig     **physical_layout_mode_config,
                             MetaMonitorsConfig     **logical_layout_mode_config,
                             GError                 **error)
 {
-  GList *logical_monitor_configs_copy, *disabled_monitor_specs_copy;
+  GList *logical_monitor_configs_copy;
+  GList *disabled_monitor_specs_copy;
+  GList *for_lease_monitor_specs_copy;
   MetaMonitorsConfig *physical_config, *logical_config;
   g_autoptr (GError) local_error_physical = NULL;
   g_autoptr (GError) local_error_logical = NULL;
@@ -743,12 +797,15 @@ detect_layout_mode_configs (MetaMonitorManager      *monitor_manager,
     meta_clone_logical_monitor_config_list (logical_monitor_configs);
   disabled_monitor_specs_copy =
     g_list_copy_deep (disabled_monitor_specs, (GCopyFunc) meta_monitor_spec_clone, NULL);
+  for_lease_monitor_specs_copy =
+    g_list_copy_deep (for_lease_monitor_specs, (GCopyFunc) meta_monitor_spec_clone, NULL);
 
   derive_logical_monitor_layouts (logical_monitor_configs,
                                   META_LOGICAL_MONITOR_LAYOUT_MODE_PHYSICAL);
   physical_config =
     meta_monitors_config_new_full (g_steal_pointer (&logical_monitor_configs),
                                    g_steal_pointer (&disabled_monitor_specs),
+                                   g_steal_pointer (&for_lease_monitor_specs),
                                    META_LOGICAL_MONITOR_LAYOUT_MODE_PHYSICAL,
                                    config_flags);
 
@@ -761,6 +818,7 @@ detect_layout_mode_configs (MetaMonitorManager      *monitor_manager,
   logical_config =
     meta_monitors_config_new_full (g_steal_pointer (&logical_monitor_configs_copy),
                                    g_steal_pointer (&disabled_monitor_specs_copy),
+                                   g_steal_pointer (&for_lease_monitor_specs_copy),
                                    META_LOGICAL_MONITOR_LAYOUT_MODE_LOGICAL,
                                    config_flags);
 
@@ -1207,6 +1265,7 @@ static MetaMonitorsConfig *
 attempt_layout_mode_conversion (MetaMonitorManager     *monitor_manager,
                                 GList                  *logical_monitor_configs,
                                 GList                  *disabled_monitor_specs,
+                                GList                  *for_lease_monitor_specs,
                                 MetaMonitorsConfigFlag  config_flags)
 {
   GList *logical_monitor_configs_copy;
@@ -1251,6 +1310,9 @@ create_full_config:
                                    g_list_copy_deep (disabled_monitor_specs,
                                                      (GCopyFunc) meta_monitor_spec_clone,
                                                      NULL),
+                                   g_list_copy_deep (for_lease_monitor_specs,
+                                                     (GCopyFunc) meta_monitor_spec_clone,
+                                                     NULL),
                                    META_LOGICAL_MONITOR_LAYOUT_MODE_LOGICAL,
                                    config_flags);
 
@@ -1385,6 +1447,14 @@ handle_end_element (GMarkupParseContext  *context,
         return;
       }
 
+    case STATE_MONITOR_COLOR_MODE:
+      {
+        g_assert (g_str_equal (element_name, "colormode"));
+
+        parser->state = STATE_MONITOR;
+        return;
+      }
+
     case STATE_MONITOR:
       {
         MetaLogicalMonitorConfig *logical_monitor_config;
@@ -1438,6 +1508,14 @@ handle_end_element (GMarkupParseContext  *context,
         return;
       }
 
+    case STATE_FOR_LEASE:
+      {
+        g_assert (g_str_equal (element_name, "forlease"));
+
+        parser->state = STATE_CONFIGURATION;
+        return;
+      }
+
     case STATE_CONFIGURATION:
       {
         MetaMonitorConfigStore *store = parser->config_store;
@@ -1458,6 +1536,7 @@ handle_end_element (GMarkupParseContext  *context,
             if (!detect_layout_mode_configs (store->monitor_manager,
                                              parser->current_logical_monitor_configs,
                                              parser->current_disabled_monitor_specs,
+                                             parser->current_for_lease_monitor_specs,
                                              config_flags,
                                              &physical_layout_mode_config,
                                              &logical_layout_mode_config,
@@ -1465,11 +1544,13 @@ handle_end_element (GMarkupParseContext  *context,
               {
                 parser->current_logical_monitor_configs = NULL;
                 parser->current_disabled_monitor_specs = NULL;
+                parser->current_for_lease_monitor_specs = NULL;
                 return;
               }
 
             parser->current_logical_monitor_configs = NULL;
             parser->current_disabled_monitor_specs = NULL;
+            parser->current_for_lease_monitor_specs = NULL;
 
             if (physical_layout_mode_config)
               {
@@ -1487,6 +1568,7 @@ handle_end_element (GMarkupParseContext  *context,
                       attempt_layout_mode_conversion (store->monitor_manager,
                                                       physical_layout_mode_config->logical_monitor_configs,
                                                       physical_layout_mode_config->disabled_monitor_specs,
+                                                      physical_layout_mode_config->for_lease_monitor_specs,
                                                       config_flags);
                   }
               }
@@ -1508,11 +1590,13 @@ handle_end_element (GMarkupParseContext  *context,
             config =
               meta_monitors_config_new_full (parser->current_logical_monitor_configs,
                                              parser->current_disabled_monitor_specs,
+                                             parser->current_for_lease_monitor_specs,
                                              layout_mode,
                                              config_flags);
 
             parser->current_logical_monitor_configs = NULL;
             parser->current_disabled_monitor_specs = NULL;
+            parser->current_for_lease_monitor_specs = NULL;
 
             if (!meta_verify_monitors_config (config, store->monitor_manager,
                                               error))
@@ -1747,6 +1831,7 @@ handle_text (GMarkupParseContext *context,
     case STATE_MONITOR_MODE:
     case STATE_TRANSFORM:
     case STATE_DISABLED:
+    case STATE_FOR_LEASE:
     case STATE_POLICY:
     case STATE_STORES:
       {
@@ -1980,6 +2065,26 @@ handle_text (GMarkupParseContext *context,
         return;
       }
 
+    case STATE_MONITOR_COLOR_MODE:
+      {
+        if (text_equals (text, text_len, "default"))
+          {
+            parser->current_monitor_config->color_mode =
+              META_COLOR_MODE_DEFAULT;
+          }
+        else if (text_equals (text, text_len, "bt2100"))
+          {
+            parser->current_monitor_config->color_mode =
+              META_COLOR_MODE_BT2100;
+          }
+        else
+          {
+            g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
+                         "Invalid color mode %.*s", (int)text_len, text);
+          }
+        return;
+      }
+
     case STATE_STORE:
       {
         MetaConfigStore store;
@@ -2154,6 +2259,28 @@ append_rgb_range (GString            *buffer,
                           rgb_range_str);
 }
 
+static void
+append_color_mode (GString       *buffer,
+                   MetaColorMode  rgb_range,
+                   const char    *indentation)
+{
+  const char *color_mode_str;
+
+  switch (rgb_range)
+    {
+    case META_COLOR_MODE_BT2100:
+      color_mode_str = "bt2100";
+      break;
+    case META_COLOR_MODE_DEFAULT:
+    default:
+      return;
+    }
+
+  g_string_append_printf (buffer, "%s<colormode>%s</colormode>\n",
+                          indentation,
+                          color_mode_str);
+}
+
 static void
 append_monitors (GString *buffer,
                  GList   *monitor_configs)
@@ -2186,6 +2313,7 @@ append_monitors (GString *buffer,
       if (monitor_config->enable_underscanning)
         g_string_append (buffer, "        <underscanning>yes</underscanning>\n");
       append_rgb_range (buffer, monitor_config->rgb_range, "        ");
+      append_color_mode (buffer, monitor_config->color_mode, "        ");
 
       if (monitor_config->has_max_bpc)
         {
@@ -2323,6 +2451,18 @@ generate_config_xml (MetaMonitorConfigStore *config_store)
           g_string_append (buffer, "    </disabled>\n");
         }
 
+      if (config->for_lease_monitor_specs)
+        {
+          g_string_append (buffer, "    <forlease>\n");
+          for (l = config->for_lease_monitor_specs; l; l = l->next)
+            {
+              MetaMonitorSpec *monitor_spec = l->data;
+
+              append_monitor_spec (buffer, monitor_spec, "      ");
+            }
+          g_string_append (buffer, "    </forlease>\n");
+        }
+
       g_string_append (buffer, "  </configuration>\n");
     }
 
