@@ -12,14 +12,11 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
-#ifndef META_SCREEN_CAST_WINDOW_STREAM_SRC_H
-#define META_SCREEN_CAST_WINDOW_STREAM_SRC_H
+#pragma once
 
 #include "backends/meta-screen-cast-stream-src.h"
 
@@ -33,5 +30,3 @@ G_DECLARE_FINAL_TYPE (MetaScreenCastWindowStreamSrc,
 
 MetaScreenCastWindowStreamSrc * meta_screen_cast_window_stream_src_new (MetaScreenCastWindowStream  *window_stream,
                                                                         GError                     **error);
-
-#endif /* META_SCREEN_CAST_WINDOW_STREAM_SRC_H */
