@@ -71,6 +71,8 @@ struct _MetaDrmLeaseManager
   gulong resources_changed_handler_id;
   gulong lease_changed_handler_id;
   gulong monitors_changed_handler_id;
+  gulong backend_pause_handler_id;
+  gulong backend_resume_handler_id;
 
   /* MetaKmsDevice *kms_device */
   GList *devices;
@@ -84,6 +86,8 @@ struct _MetaDrmLeaseManager
    * Value: MetaDrmLease *lease
    */
   GHashTable *leased_connectors;
+
+  gboolean is_paused;
 };
 
 G_DEFINE_TYPE (MetaDrmLeaseManager, meta_drm_lease_manager, G_TYPE_OBJECT)
@@ -665,6 +669,9 @@ update_connectors (MetaDrmLeaseManager  *lease_manager,
   new_leased_connectors =
     g_hash_table_new_similar (lease_manager->leased_connectors);
 
+  if (lease_manager->is_paused)
+    goto scanned_resources;
+
   for (l = meta_kms_get_devices (kms); l; l = l->next)
     {
       MetaKmsDevice *kms_device = l->data;
@@ -697,12 +704,7 @@ update_connectors (MetaDrmLeaseManager  *lease_manager,
         }
     }
 
-  for (l = lease_manager->connectors; l; l = l->next)
-    {
-      kms_connector = l->data;
-
-      removed_connectors = g_list_append (removed_connectors, kms_connector);
-    }
+scanned_resources:
 
   g_hash_table_iter_init (&iter, lease_manager->leased_connectors);
   while (g_hash_table_iter_next (&iter, (gpointer *)&kms_connector, NULL))
@@ -713,7 +715,7 @@ update_connectors (MetaDrmLeaseManager  *lease_manager,
         leases_to_revoke = g_list_append (leases_to_revoke, lease);
     }
 
-  g_list_free (g_steal_pointer (&lease_manager->connectors));
+  removed_connectors = g_steal_pointer (&lease_manager->connectors);
   lease_manager->connectors = new_connectors;
 
   g_clear_pointer (&lease_manager->leased_connectors, g_hash_table_unref);
@@ -869,6 +871,21 @@ on_lease_changed (MetaKms             *kms,
   update_leases (lease_manager);
 }
 
+static void
+on_pause (MetaBackend         *backend,
+          MetaDrmLeaseManager *lease_manager)
+{
+  lease_manager->is_paused = TRUE;
+  update_resources (lease_manager);
+}
+
+static void
+on_resume (MetaBackend         *backend,
+           MetaDrmLeaseManager *lease_manager)
+{
+  lease_manager->is_paused = FALSE;
+}
+
 static void
 meta_drm_lease_manager_constructed (GObject *object)
 {
@@ -878,10 +895,12 @@ meta_drm_lease_manager_constructed (GObject *object)
   MetaMonitorManager *monitor_manager =
     meta_backend_get_monitor_manager (backend);
 
+  /* Connect to MetaKms::resources-changed using G_CONNECT_AFTER to make sure
+   * MetaMonitorManager state is up to date. */
   lease_manager->resources_changed_handler_id =
-    g_signal_connect (kms, "resources-changed",
-                      G_CALLBACK (on_resources_changed),
-                      lease_manager);
+    g_signal_connect_after (kms, "resources-changed",
+                            G_CALLBACK (on_resources_changed),
+                            lease_manager);
   lease_manager->lease_changed_handler_id =
     g_signal_connect (kms, "lease-changed",
                       G_CALLBACK (on_lease_changed),
@@ -890,6 +909,10 @@ meta_drm_lease_manager_constructed (GObject *object)
     g_signal_connect_swapped (monitor_manager, "monitors-changed-internal",
                               G_CALLBACK (update_resources),
                               lease_manager);
+  lease_manager->backend_pause_handler_id =
+    g_signal_connect (backend, "pause", G_CALLBACK (on_pause), lease_manager);
+  lease_manager->backend_resume_handler_id =
+    g_signal_connect (backend, "resume", G_CALLBACK (on_resume), lease_manager);
 
   lease_manager->leases =
     g_hash_table_new_full (NULL, NULL,
@@ -949,6 +972,10 @@ meta_drm_lease_manager_dispose (GObject *object)
   g_clear_signal_handler (&lease_manager->lease_changed_handler_id, kms);
   g_clear_signal_handler (&lease_manager->monitors_changed_handler_id,
                           monitor_manager);
+  g_clear_signal_handler (&lease_manager->backend_pause_handler_id,
+                          backend);
+  g_clear_signal_handler (&lease_manager->backend_resume_handler_id,
+                          backend);
 
   g_list_free_full (g_steal_pointer (&lease_manager->devices), g_object_unref);
   g_list_free_full (g_steal_pointer (&lease_manager->connectors),
