@@ -27,10 +27,6 @@ MetaKmsResourceChanges meta_kms_device_update_states_in_impl (MetaKmsDevice *dev
                                                               uint32_t       crtc_id,
                                                               uint32_t       connector_id);
 
-void meta_kms_device_add_fake_plane_in_impl (MetaKmsDevice    *device,
-                                             MetaKmsPlaneType  plane_type,
-                                             MetaKmsCrtc      *crtc);
-
 MetaKmsCrtc * meta_kms_device_find_crtc_in_impl (MetaKmsDevice *device,
                                                  uint32_t       crtc_id);
 
