@@ -32,6 +32,21 @@ enum _MetaKmsPlaneType
   META_KMS_PLANE_TYPE_OVERLAY,
 };
 
+typedef enum _MetaKmsPlaneYCbCrColorEncoding
+{
+  META_KMS_PLANE_YCBCR_COLOR_ENCODING_BT601 = 0,
+  META_KMS_PLANE_YCBCR_COLOR_ENCODING_BT709,
+  META_KMS_PLANE_YCBCR_COLOR_ENCODING_BT2020,
+  META_KMS_PLANE_YCBCR_COLOR_ENCODING_N_PROPS,
+} MetaKmsPlaneYCbCrColorEncoding;
+
+typedef enum _MetaKmsPlaneYCbCrColorRanges
+{
+  META_KMS_PLANE_YCBCR_COLOR_RANGE_LIMITED = 0,
+  META_KMS_PLANE_YCBCR_COLOR_RANGE_FULL,
+  META_KMS_PLANE_YCBCR_COLOR_RANGE_N_PROPS,
+} MetaKmsPlaneYCbCrColorRange;
+
 typedef struct _MetaKmsPlaneCursorSizeHints
 {
   gboolean has_size_hints;
@@ -59,6 +74,12 @@ meta_kms_plane_get_cursor_size_hints (MetaKmsPlane *plane);
 gboolean meta_kms_plane_is_transform_handled (MetaKmsPlane        *plane,
                                               MtkMonitorTransform  transform);
 
+gboolean meta_kms_plane_is_color_encoding_handled (MetaKmsPlane                   *plane,
+                                                   MetaKmsPlaneYCbCrColorEncoding  encoding);
+
+gboolean meta_kms_plane_is_color_range_handled (MetaKmsPlane                *plane,
+                                                MetaKmsPlaneYCbCrColorRange  range);
+
 gboolean meta_kms_plane_supports_cursor_hotspot (MetaKmsPlane *plane);
 
 GArray * meta_kms_plane_get_modifiers_for_format (MetaKmsPlane *plane,
@@ -77,4 +98,12 @@ void meta_kms_plane_update_set_rotation (MetaKmsPlane           *plane,
                                          MetaKmsPlaneAssignment *plane_assignment,
                                          MtkMonitorTransform     transform);
 
+void meta_kms_plane_update_set_color_encoding (MetaKmsPlane                   *plane,
+                                               MetaKmsPlaneAssignment         *plane_assignment,
+                                               MetaKmsPlaneYCbCrColorEncoding  encoding);
+
+void meta_kms_plane_update_set_color_range (MetaKmsPlane                *plane,
+                                            MetaKmsPlaneAssignment      *plane_assignment,
+                                            MetaKmsPlaneYCbCrColorRange  range);
+
 const char * meta_kms_plane_type_to_string (MetaKmsPlaneType plane_type);
