@@ -39,8 +39,7 @@ gpointer meta_kms_run_impl_task_sync (MetaKms             *kms,
                                       GError             **error);
 
 META_EXPORT_TEST
-MetaKmsResourceChanges meta_kms_update_states_sync (MetaKms     *kms,
-                                                    GUdevDevice *udev_device);
+MetaKmsResourceChanges meta_kms_update_states_sync (MetaKms *kms);
 
 gboolean meta_kms_in_impl_task (MetaKms *kms);
 
