@@ -36,7 +36,8 @@ G_DECLARE_FINAL_TYPE (MetaPointerConstraintImplNative,
                       MetaPointerConstraintImpl)
 
 MetaPointerConstraintImpl * meta_pointer_constraint_impl_native_new (MetaPointerConstraint *constraint_impl,
-                                                                     const cairo_region_t  *region,
+                                                                     const MtkRegion       *region,
+                                                                     graphene_point_t       origin,
                                                                      double                 min_edge_distance);
 
 G_END_DECLS
