@@ -12,14 +12,11 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  */
 
-#ifndef META_RENDER_DEVICE_GBM_H
-#define META_RENDER_DEVICE_GBM_H
+#pragma once
 
 #include "backends/native/meta-render-device-private.h"
 
@@ -33,5 +30,3 @@ MetaRenderDeviceGbm * meta_render_device_gbm_new (MetaBackend     *backend,
                                                   GError         **error);
 
 struct gbm_device * meta_render_device_gbm_get_gbm_device (MetaRenderDeviceGbm *render_device_gbm);
-
-#endif /* META_RENDER_DEVICE_GBM_H */
