@@ -14,21 +14,21 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  * Written by:
  *     Jasper St. Pierre <jstpierre@mecheye.net>
  */
 
-#ifndef META_BACKEND_X11_H
-#define META_BACKEND_X11_H
+#pragma once
 
 #include <stdint.h>
 #include <X11/Xlib.h>
+#include <X11/extensions/XInput.h>
+#include <X11/extensions/XInput2.h>
 
 #include "backends/meta-backend-private.h"
+#include "backends/x11/meta-backend-x11-types.h"
 #include "backends/x11/meta-clutter-backend-x11.h"
 
 #define META_TYPE_BACKEND_X11 (meta_backend_x11_get_type ())
@@ -55,13 +55,12 @@ Window meta_backend_x11_get_root_xwindow (MetaBackendX11 *backend_x11);
 
 Window meta_backend_x11_get_xwindow (MetaBackendX11 *backend);
 
-void meta_backend_x11_handle_event (MetaBackendX11 *x11,
-                                    XEvent         *xevent);
-
 uint8_t meta_backend_x11_get_xkb_event_base (MetaBackendX11 *x11);
 
 void meta_backend_x11_reload_cursor (MetaBackendX11 *x11);
 
 void meta_backend_x11_sync_pointer (MetaBackendX11 *backend_x11);
 
-#endif /* META_BACKEND_X11_H */
+MetaX11Barriers * meta_backend_x11_get_barriers (MetaBackendX11 *backend_x11);
+
+void meta_backend_x11_reset_cached_logical_monitor (MetaBackendX11 *backend_x11);
