@@ -23,13 +23,7 @@
 #include "core/util-private.h"
 
 #define META_TYPE_BACKEND_X11_NESTED (meta_backend_x11_nested_get_type ())
-META_EXPORT_TEST
-G_DECLARE_DERIVABLE_TYPE (MetaBackendX11Nested, meta_backend_x11_nested,
-                          META, BACKEND_X11_NESTED, MetaBackendX11)
-
-struct _MetaBackendX11NestedClass
-{
-  MetaBackendX11Class parent_class;
-
-  void (* init_gpus) (MetaBackendX11Nested *backend_x11_nested);
-};
+G_DECLARE_FINAL_TYPE (MetaBackendX11Nested,
+                      meta_backend_x11_nested,
+                      META, BACKEND_X11_NESTED,
+                      MetaBackendX11)
