@@ -344,6 +344,7 @@ try_assign_next_scanout (MetaCompositorView *compositor_view,
       return;
     }
 
+  meta_topic (META_DEBUG_RENDER, "Assigning scanout to stage view");
   clutter_stage_view_assign_next_scanout (stage_view, scanout);
 }
 
@@ -377,11 +378,12 @@ static MetaSurfaceActor *
 find_frame_sync_candidate (MetaCompositorView *compositor_view,
                            MetaCompositor     *compositor)
 {
+  ClutterStageView *stage_view =
+    meta_compositor_view_get_stage_view (compositor_view);
   MetaWindowActor *window_actor;
-  MetaWindow *window;
-  ClutterStageView *stage_view;
-  MtkRectangle view_layout;
   MetaSurfaceActor *surface_actor;
+  MtkRectangle view_rect;
+  ClutterActorBox actor_box;
 
   if (meta_compositor_is_unredirect_inhibited (compositor))
     {
@@ -420,23 +422,31 @@ find_frame_sync_candidate (MetaCompositorView *compositor_view,
       return NULL;
     }
 
-  window = meta_window_actor_get_meta_window (window_actor);
-  if (!window)
+  clutter_stage_view_get_layout (stage_view, &view_rect);
+
+  if (!clutter_actor_get_paint_box (CLUTTER_ACTOR (window_actor),
+                                    &actor_box))
     {
       meta_topic (META_DEBUG_RENDER,
-                  "No frame sync candidate: no meta-window");
+                  "No frame sync candidate: no window actor paint-box");
       return NULL;
     }
 
-  stage_view = meta_compositor_view_get_stage_view (compositor_view);
-
-  clutter_stage_view_get_layout (stage_view, &view_layout);
-
-  if (!meta_window_geometry_contains_rect (window, &view_layout))
+  if (!G_APPROX_VALUE (actor_box.x1, view_rect.x,
+                       CLUTTER_COORDINATE_EPSILON) ||
+      !G_APPROX_VALUE (actor_box.y1, view_rect.y,
+                       CLUTTER_COORDINATE_EPSILON) ||
+      !G_APPROX_VALUE (actor_box.x2, view_rect.x + view_rect.width,
+                       CLUTTER_COORDINATE_EPSILON) ||
+      !G_APPROX_VALUE (actor_box.y2, view_rect.y + view_rect.height,
+                       CLUTTER_COORDINATE_EPSILON))
     {
       meta_topic (META_DEBUG_RENDER,
-                  "No frame sync candidate: stage-view layout not covered "
-                  "by meta-window frame");
+                  "No frame sync candidate: paint-box (%f,%f,%f,%f) does "
+                  "not match stage-view layout (%d,%d,%d,%d)",
+                  actor_box.x1, actor_box.y1,
+                  actor_box.x2 - actor_box.x1, actor_box.y2 - actor_box.y1,
+                  view_rect.x, view_rect.y, view_rect.width, view_rect.height);
       return NULL;
     }
 
@@ -448,6 +458,13 @@ find_frame_sync_candidate (MetaCompositorView *compositor_view,
       return NULL;
     }
 
+  if (meta_surface_actor_is_effectively_obscured (surface_actor))
+    {
+      meta_topic (META_DEBUG_RENDER,
+                  "No frame sync candidate: surface-actor is obscured");
+      return NULL;
+    }
+
   if (meta_surface_actor_is_frozen (surface_actor))
     {
       meta_topic (META_DEBUG_RENDER,
@@ -455,15 +472,6 @@ find_frame_sync_candidate (MetaCompositorView *compositor_view,
       return NULL;
     }
 
-  if (!meta_surface_actor_contains_rect (surface_actor,
-                                         &view_layout))
-    {
-      meta_topic (META_DEBUG_RENDER,
-                  "No frame sync candidate: stage-view layout not covered "
-                  "by surface-actor");
-      return NULL;
-    }
-
   return surface_actor;
 }
 
