@@ -48,13 +48,10 @@ G_DEFINE_TYPE (MetaSurfaceActorWayland,
                META_TYPE_SURFACE_ACTOR)
 
 static void
-meta_surface_actor_wayland_process_damage (MetaSurfaceActor *actor,
-                                           int               x,
-                                           int               y,
-                                           int               width,
-                                           int               height)
+meta_surface_actor_wayland_process_damage (MetaSurfaceActor   *actor,
+                                           const MtkRectangle *area)
 {
-  meta_surface_actor_update_area (actor, x, y, width, height);
+  meta_surface_actor_update_area (actor, area);
 }
 
 static gboolean
@@ -260,6 +257,18 @@ out:
   parent_class->apply_transform (actor, matrix);
 }
 
+static void
+on_surface_disposed (gpointer user_data,
+                     GObject *destroyed_object)
+{
+  MetaSurfaceActorWayland *self = user_data;
+
+  g_assert (destroyed_object == (GObject *) self->surface);
+
+  clutter_actor_set_reactive (CLUTTER_ACTOR (self), FALSE);
+  self->surface = NULL;
+}
+
 static void
 meta_surface_actor_wayland_dispose (GObject *object)
 {
@@ -272,8 +281,9 @@ meta_surface_actor_wayland_dispose (GObject *object)
 
   if (self->surface)
     {
-      g_object_remove_weak_pointer (G_OBJECT (self->surface),
-                                    (gpointer *) &self->surface);
+      g_object_weak_unref (G_OBJECT (self->surface),
+                           on_surface_disposed,
+                           self);
       self->surface = NULL;
     }
 
@@ -303,13 +313,16 @@ meta_surface_actor_wayland_init (MetaSurfaceActorWayland *self)
 MetaSurfaceActor *
 meta_surface_actor_wayland_new (MetaWaylandSurface *surface)
 {
-  MetaSurfaceActorWayland *self = g_object_new (META_TYPE_SURFACE_ACTOR_WAYLAND, NULL);
+  MetaSurfaceActorWayland *self = g_object_new (META_TYPE_SURFACE_ACTOR_WAYLAND,
+                                                "accessible-name", "Wayland surface",
+                                                NULL);
 
   g_assert (meta_is_wayland_compositor ());
 
   self->surface = surface;
-  g_object_add_weak_pointer (G_OBJECT (self->surface),
-                             (gpointer *) &self->surface);
+  g_object_weak_ref (G_OBJECT (self->surface),
+                     on_surface_disposed,
+                     self);
 
   return META_SURFACE_ACTOR (self);
 }
