@@ -139,7 +139,8 @@ load_gl_symbol (CoglContext *ctx,
   *func = cogl_renderer_get_proc_address (ctx->display->renderer, name);
   if (!*func)
     {
-      meta_verbose ("MetaSyncRing: failed to resolve required GL symbol \"%s\"", name);
+      meta_topic (META_DEBUG_RENDER,
+                  "MetaSyncRing: failed to resolve required GL symbol \"%s\"", name);
       return FALSE;
     }
   return TRUE;
@@ -154,9 +155,9 @@ check_gl_extensions (CoglContext *cogl_context)
   cogl_display = cogl_context_get_display (cogl_context);
   cogl_renderer = cogl_display_get_renderer (cogl_display);
 
-  switch (cogl_renderer_get_driver (cogl_renderer))
+  switch (cogl_renderer->driver_id)
     {
-    case COGL_DRIVER_GL3:
+    case COGL_DRIVER_ID_GL3:
       {
         int num_extensions, i;
         gboolean arb_sync = FALSE;
@@ -205,7 +206,8 @@ load_required_symbols (CoglContext *ctx)
 
   if (!check_gl_extensions (ctx))
     {
-      meta_verbose ("MetaSyncRing: couldn't find required GL extensions");
+      meta_topic (META_DEBUG_RENDER,
+                  "MetaSyncRing: couldn't find required GL extensions");
       goto out;
     }
 
@@ -489,7 +491,7 @@ meta_sync_ring_reboot (CoglContext *ctx,
 
   if (!meta_sync_ring_get ())
     {
-      meta_warning ("MetaSyncRing: Too many reboots -- disabling");
+      g_warning ("MetaSyncRing: Too many reboots -- disabling");
       return FALSE;
     }
 
@@ -514,13 +516,13 @@ meta_sync_ring_after_frame (CoglContext *ctx)
       GLenum status = meta_sync_check_update_finished (sync_to_reset, 0);
       if (status == GL_TIMEOUT_EXPIRED)
         {
-          meta_warning ("MetaSyncRing: We should never wait for a sync -- add more syncs?");
+          g_warning ("MetaSyncRing: We should never wait for a sync -- add more syncs?");
           status = meta_sync_check_update_finished (sync_to_reset, MAX_SYNC_WAIT_TIME);
         }
 
       if (status != GL_ALREADY_SIGNALED && status != GL_CONDITION_SATISFIED)
         {
-          meta_warning ("MetaSyncRing: Timed out waiting for sync object.");
+          g_warning ("MetaSyncRing: Timed out waiting for sync object.");
           return meta_sync_ring_reboot (ctx, ring->xdisplay);
         }
 
@@ -560,7 +562,7 @@ meta_sync_ring_insert_wait (CoglContext *ctx)
     }
   else if (sync->state != META_SYNC_STATE_READY)
     {
-      meta_warning ("MetaSyncRing: Sync object is not ready -- were events handled properly?");
+      g_warning ("MetaSyncRing: Sync object is not ready -- were events handled properly?");
       if (!meta_sync_ring_reboot (ctx, ring->xdisplay))
         return FALSE;
     }
