@@ -20,8 +20,7 @@
  * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef META_EVENTS_H
-#define META_EVENTS_H
+#pragma once
 
 #include "meta/display.h"
 
@@ -35,5 +34,3 @@
 
 void meta_display_init_events (MetaDisplay *display);
 void meta_display_free_events (MetaDisplay *display);
-
-#endif
