@@ -15,21 +15,30 @@
  * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef META_KEYBINDINGS_H
-#define META_KEYBINDINGS_H
+#pragma once
 
-#include <meta/display.h>
-#include <meta/common.h>
+#include "meta/display.h"
+#include "meta/common.h"
 
 #define META_TYPE_KEY_BINDING               (meta_key_binding_get_type ())
 
+META_EXPORT
 const char          *meta_key_binding_get_name      (MetaKeyBinding *binding);
-MetaVirtualModifier  meta_key_binding_get_modifiers (MetaKeyBinding *binding);
+
+META_EXPORT
+ClutterModifierType  meta_key_binding_get_modifiers (MetaKeyBinding *binding);
+
+META_EXPORT
 guint                meta_key_binding_get_mask      (MetaKeyBinding *binding);
+
+META_EXPORT
 gboolean             meta_key_binding_is_builtin    (MetaKeyBinding *binding);
 
+META_EXPORT
+gboolean             meta_key_binding_is_reversed   (MetaKeyBinding *binding);
+
+META_EXPORT
 gboolean meta_keybindings_set_custom_handler (const gchar        *name,
 					      MetaKeyHandlerFunc  handler,
 					      gpointer            user_data,
 					      GDestroyNotify      free_data);
-#endif
