@@ -19,15 +19,16 @@
  * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef META_MAIN_H
-#define META_MAIN_H
+#pragma once
 
 #include <glib.h>
 
-#include <meta/common.h>
+#include "meta/common.h"
+#include "meta/meta-context.h"
 
 META_EXPORT
-void            meta_restart                (const char *message);
+void            meta_restart                (const char  *message,
+                                             MetaContext *context);
 
 META_EXPORT
 gboolean        meta_is_restart             (void);
@@ -46,5 +47,3 @@ typedef enum
 /* exit immediately */
 META_EXPORT
 void meta_exit (MetaExitCode code) G_GNUC_NORETURN;
-
-#endif
