@@ -14,30 +14,26 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef META_TYPES_H
-#define META_TYPES_H
+#pragma once
 
-/**
- * MetaCompositor: (skip)
- *
- */
+typedef struct _MetaBackend     MetaBackend;
+typedef struct _MetaContext     MetaContext;
 typedef struct _MetaCompositor  MetaCompositor;
 typedef struct _MetaDisplay     MetaDisplay;
-typedef struct _MetaFrame       MetaFrame;
-typedef struct _MetaScreen      MetaScreen;
 typedef struct _MetaWindow      MetaWindow;
 typedef struct _MetaWorkspace   MetaWorkspace;
-/**
- * MetaGroup: (skip)
- *
- */
-typedef struct _MetaGroup       MetaGroup;
+typedef struct _MetaLaters      MetaLaters;
+
 typedef struct _MetaKeyBinding  MetaKeyBinding;
 typedef struct _MetaCursorTracker MetaCursorTracker;
 
-#endif
+typedef struct _MetaDnd         MetaDnd;
+typedef struct _MetaSettings    MetaSettings;
+
+typedef struct _MetaWorkspaceManager MetaWorkspaceManager;
+typedef struct _MetaSelection MetaSelection;
+typedef struct _MetaDebugControl MetaDebugControl;
+typedef struct _MetaWindowConfig MetaWindowConfig;
