@@ -29,14 +29,17 @@ actor_color_state_default (void)
 {
   ClutterActor *actor;
   ClutterColorState *color_state;
-  ClutterColorspace colorspace;
+  ClutterColorStateParams *color_state_params;
+  const ClutterColorimetry *colorimetry;
 
   actor = clutter_actor_new ();
 
   color_state = clutter_actor_get_color_state (actor);
-  colorspace = clutter_color_state_get_colorspace (color_state);
+  color_state_params = CLUTTER_COLOR_STATE_PARAMS (color_state);
+  colorimetry = clutter_color_state_params_get_colorimetry (color_state_params);
 
-  g_assert_cmpuint (colorspace, ==, CLUTTER_COLORSPACE_DEFAULT);
+  g_assert_cmpuint (colorimetry->type, ==, CLUTTER_COLORIMETRY_TYPE_COLORSPACE);
+  g_assert_cmpuint (colorimetry->colorspace, ==, CLUTTER_COLORSPACE_SRGB);
 
   clutter_actor_destroy (actor);
 }
@@ -49,12 +52,13 @@ actor_color_state_passed (void)
   ClutterContext *context = clutter_test_get_context ();
   ClutterActor *actor;
   ClutterColorState *color_state;
-  ClutterColorspace colorspace;
-  ClutterTransferFunction transfer_function;
+  ClutterColorStateParams *color_state_params;
+  const ClutterColorimetry *colorimetry;
+  const ClutterEOTF *eotf;
 
-  color_state = clutter_color_state_new (context,
-                                         CLUTTER_COLORSPACE_BT2020,
-                                         CLUTTER_TRANSFER_FUNCTION_PQ);
+  color_state = clutter_color_state_params_new (context,
+                                                CLUTTER_COLORSPACE_BT2020,
+                                                CLUTTER_TRANSFER_FUNCTION_PQ);
 
   if (!color_state)
     g_critical ("Failed to create color state with provided colorspace.");
@@ -70,11 +74,14 @@ actor_color_state_passed (void)
     g_critical ("Failed to create actor with provided color state.");
 
   color_state = clutter_actor_get_color_state (actor);
-  colorspace = clutter_color_state_get_colorspace (color_state);
-  transfer_function = clutter_color_state_get_transfer_function (color_state);
+  color_state_params = CLUTTER_COLOR_STATE_PARAMS (color_state);
+  colorimetry = clutter_color_state_params_get_colorimetry (color_state_params);
+  eotf = clutter_color_state_params_get_eotf (color_state_params);
 
-  g_assert_cmpuint (colorspace, ==, CLUTTER_COLORSPACE_BT2020);
-  g_assert_cmpuint (transfer_function, ==, CLUTTER_TRANSFER_FUNCTION_PQ);
+  g_assert_cmpuint (colorimetry->type, ==, CLUTTER_COLORIMETRY_TYPE_COLORSPACE);
+  g_assert_cmpuint (colorimetry->colorspace, ==, CLUTTER_COLORSPACE_BT2020);
+  g_assert_cmpuint (eotf->type, ==, CLUTTER_EOTF_TYPE_NAMED);
+  g_assert_cmpuint (eotf->tf_name, ==, CLUTTER_TRANSFER_FUNCTION_PQ);
 
   clutter_actor_destroy (actor);
 }
@@ -86,14 +93,15 @@ actor_change_color_state (void)
   ClutterContext *context = clutter_test_get_context ();
   ClutterActor *actor;
   ClutterColorState *color_state;
-  ClutterColorspace colorspace;
-  ClutterTransferFunction transfer_function;
+  ClutterColorStateParams *color_state_params;
+  const ClutterColorimetry *colorimetry;
+  const ClutterEOTF *eotf;
 
   actor = clutter_actor_new ();
 
-  color_state = clutter_color_state_new (context,
-                                         CLUTTER_COLORSPACE_BT2020,
-                                         CLUTTER_TRANSFER_FUNCTION_PQ);
+  color_state = clutter_color_state_params_new (context,
+                                                CLUTTER_COLORSPACE_BT2020,
+                                                CLUTTER_TRANSFER_FUNCTION_PQ);
 
   if (!color_state)
     g_critical ("Failed to create color state with provided colorspace.");
@@ -101,11 +109,14 @@ actor_change_color_state (void)
   clutter_actor_set_color_state (actor, color_state);
 
   color_state = clutter_actor_get_color_state (actor);
-  colorspace = clutter_color_state_get_colorspace (color_state);
-  transfer_function = clutter_color_state_get_transfer_function (color_state);
+  color_state_params = CLUTTER_COLOR_STATE_PARAMS (color_state);
+  colorimetry = clutter_color_state_params_get_colorimetry (color_state_params);
+  eotf = clutter_color_state_params_get_eotf (color_state_params);
 
-  g_assert_cmpuint (colorspace, ==, CLUTTER_COLORSPACE_BT2020);
-  g_assert_cmpuint (transfer_function, ==, CLUTTER_TRANSFER_FUNCTION_PQ);
+  g_assert_cmpuint (colorimetry->type, ==, CLUTTER_COLORIMETRY_TYPE_COLORSPACE);
+  g_assert_cmpuint (colorimetry->colorspace, ==, CLUTTER_COLORSPACE_BT2020);
+  g_assert_cmpuint (eotf->type, ==, CLUTTER_EOTF_TYPE_NAMED);
+  g_assert_cmpuint (eotf->tf_name, ==, CLUTTER_TRANSFER_FUNCTION_PQ);
 
   clutter_actor_destroy (actor);
 }
@@ -115,19 +126,23 @@ actor_unset_color_state (void)
 {
   ClutterActor *actor;
   ClutterColorState *color_state;
-  ClutterColorspace colorspace;
-  ClutterTransferFunction transfer_function;
+  ClutterColorStateParams *color_state_params;
+  const ClutterColorimetry *colorimetry;
+  const ClutterEOTF *eotf;
 
   actor = clutter_actor_new ();
 
   clutter_actor_unset_color_state (actor);
 
   color_state = clutter_actor_get_color_state (actor);
-  colorspace = clutter_color_state_get_colorspace (color_state);
-  transfer_function = clutter_color_state_get_transfer_function (color_state);
+  color_state_params = CLUTTER_COLOR_STATE_PARAMS (color_state);
+  colorimetry = clutter_color_state_params_get_colorimetry (color_state_params);
+  eotf = clutter_color_state_params_get_eotf (color_state_params);
 
-  g_assert_cmpuint (colorspace, ==, CLUTTER_COLORSPACE_DEFAULT);
-  g_assert_cmpuint (transfer_function, ==, CLUTTER_TRANSFER_FUNCTION_DEFAULT);
+  g_assert_cmpuint (colorimetry->type, ==, CLUTTER_COLORIMETRY_TYPE_COLORSPACE);
+  g_assert_cmpuint (colorimetry->colorspace, ==, CLUTTER_COLORSPACE_SRGB);
+  g_assert_cmpuint (eotf->type, ==, CLUTTER_EOTF_TYPE_NAMED);
+  g_assert_cmpuint (eotf->tf_name, ==, CLUTTER_TRANSFER_FUNCTION_SRGB);
 
   clutter_actor_destroy (actor);
 }
