@@ -92,9 +92,9 @@ key_group_action_activate (KeyGroup            *self,
   ClutterActor *child = NULL;
 
   g_assert_cmpstr (action_name, ==, "activate");
-  g_assert (key_val == CLUTTER_KEY_Return ||
-            key_val == CLUTTER_KEY_KP_Enter ||
-            key_val == CLUTTER_KEY_ISO_Enter);
+  g_assert_true (key_val == CLUTTER_KEY_Return ||
+                 key_val == CLUTTER_KEY_KP_Enter ||
+                 key_val == CLUTTER_KEY_ISO_Enter);
 
   if (self->selected_index == -1)
     return FALSE;
@@ -117,7 +117,7 @@ key_group_key_press (ClutterActor *actor,
   gboolean res;
 
   pool = clutter_binding_pool_find (G_OBJECT_TYPE_NAME (actor));
-  g_assert (pool != NULL);
+  g_assert_nonnull (pool);
 
   res = clutter_binding_pool_activate (pool,
                                        clutter_event_get_key_symbol (event),
