@@ -200,10 +200,10 @@ G_MODULE_EXPORT int
 test_grab_main (int argc, char *argv[])
 {
   ClutterActor   *stage, *actor;
-  ClutterColor    rcol = { 0xff, 0, 0, 0xff},
-                  bcol = { 0, 0, 0xff, 0xff },
-                  ccol = { 0, 0xff, 0xff, 0xff },
-                  ycol = { 0xff, 0xff, 0, 0xff };
+  CoglColor rcol = { 0xff, 0, 0, 0xff},
+            bcol = { 0, 0, 0xff, 0xff },
+            ccol = { 0, 0xff, 0xff, 0xff },
+            ycol = { 0xff, 0xff, 0, 0xff };
 
   clutter_test_init (&argc, &argv);
 
@@ -214,7 +214,6 @@ test_grab_main (int argc, char *argv[])
   g_print ("Cyan  box:  toggle grab (from cyan box) for keyboard events.\n\n");
 
   stage = clutter_test_get_stage ();
-  clutter_stage_set_title (CLUTTER_STAGE (stage), "Grabs");
   g_signal_connect (stage, "destroy", G_CALLBACK (clutter_test_quit), NULL);
   g_signal_connect (stage, "event",
                     G_CALLBACK (debug_event_cb), (char *) "stage");
