@@ -33,12 +33,12 @@
 #include "meta/compositor.h"
 #include "meta/meta-context.h"
 #include "tests/boxes-tests.h"
-#include "tests/monitor-config-migration-unit-tests.h"
 #include "tests/monitor-store-unit-tests.h"
 #include "tests/monitor-transform-tests.h"
 #include "tests/meta-test-utils.h"
 #include "tests/orientation-manager-unit-tests.h"
 #include "tests/hdr-metadata-unit-tests.h"
+#include "tests/button-transform-tests.h"
 
 MetaContext *test_context;
 
@@ -199,36 +199,6 @@ meta_test_util_later_schedule_from_later (void)
   g_assert_cmpint (data.state, ==, META_TEST_LATER_FINISHED);
 }
 
-static void
-meta_test_adjacent_to (void)
-{
-  MetaRectangle base = { .x = 10, .y = 10, .width = 10, .height = 10 };
-  MetaRectangle adjacent[] = {
-    { .x = 20, .y = 10, .width = 10, .height = 10 },
-    { .x = 0, .y = 10, .width = 10, .height = 10 },
-    { .x = 0, .y = 1, .width = 10, .height = 10 },
-    { .x = 20, .y = 19, .width = 10, .height = 10 },
-    { .x = 10, .y = 20, .width = 10, .height = 10 },
-    { .x = 10, .y = 0, .width = 10, .height = 10 },
-  };
-  MetaRectangle not_adjacent[] = {
-    { .x = 0, .y = 0, .width = 10, .height = 10 },
-    { .x = 20, .y = 20, .width = 10, .height = 10 },
-    { .x = 21, .y = 10, .width = 10, .height = 10 },
-    { .x = 10, .y = 21, .width = 10, .height = 10 },
-    { .x = 10, .y = 5, .width = 10, .height = 10 },
-    { .x = 11, .y = 10, .width = 10, .height = 10 },
-    { .x = 19, .y = 10, .width = 10, .height = 10 },
-  };
-  unsigned int i;
-
-  for (i = 0; i < G_N_ELEMENTS (adjacent); i++)
-    g_assert (meta_rectangle_is_adjacent_to (&base, &adjacent[i]));
-
-  for (i = 0; i < G_N_ELEMENTS (not_adjacent); i++)
-    g_assert (!meta_rectangle_is_adjacent_to (&base, &not_adjacent[i]));
-}
-
 static void
 init_tests (void)
 {
@@ -236,14 +206,12 @@ init_tests (void)
   g_test_add_func ("/util/meta-later/schedule-from-later",
                    meta_test_util_later_schedule_from_later);
 
-  g_test_add_func ("/core/boxes/adjacent-to", meta_test_adjacent_to);
-
   init_monitor_store_tests ();
-  init_monitor_config_migration_tests ();
   init_boxes_tests ();
   init_monitor_transform_tests ();
   init_orientation_manager_tests ();
   init_hdr_metadata_tests ();
+  init_button_transform_tests ();
 }
 
 int
@@ -252,7 +220,7 @@ main (int argc, char *argv[])
   g_autoptr (MetaContext) context = NULL;
   g_autoptr (GError) error = NULL;
 
-  context = meta_create_test_context (META_CONTEXT_TEST_TYPE_NESTED,
+  context = meta_create_test_context (META_CONTEXT_TEST_TYPE_TEST,
                                       META_CONTEXT_TEST_FLAG_TEST_CLIENT);
   if (!meta_context_configure (context, &argc, &argv, &error))
     g_error ("Failed to configure test context: %s", error->message);
