@@ -314,6 +314,13 @@ static const struct wl_seat_listener wl_seat_listener = {
   handle_wl_seat_name,
 };
 
+static void
+test_driver_handle_terminate (void               *user_data,
+                              struct test_driver *test_driver)
+{
+  exit (EXIT_SUCCESS);
+}
+
 static void
 test_driver_handle_sync_event (void               *user_data,
                                struct test_driver *test_driver,
@@ -351,6 +358,7 @@ test_driver_handle_property_int (void               *user_data,
 }
 
 static const struct test_driver_listener test_driver_listener = {
+  test_driver_handle_terminate,
   test_driver_handle_sync_event,
   test_driver_handle_property,
   test_driver_handle_property_int,
@@ -663,15 +671,23 @@ handle_xdg_toplevel_configure (void                *data,
   WaylandSurface *surface = data;
   uint32_t *p;
 
-  if (width == 0)
-    surface->width = surface->default_width;
+  if (surface->fixed_size)
+    {
+      surface->width = surface->default_width;
+      surface->height = surface->default_height;
+    }
   else
-    surface->width = width;
+    {
+      if (width == 0)
+        surface->width = surface->default_width;
+      else
+        surface->width = width;
 
-  if (height == 0)
-    surface->height = surface->default_height;
-  else
-    surface->height = height;
+      if (height == 0)
+        surface->height = surface->default_height;
+      else
+        surface->height = height;
+    }
 
   g_assert_null (surface->pending_state);
   surface->pending_state = g_hash_table_new (NULL, NULL);
@@ -888,6 +904,18 @@ wayland_surface_set_opaque (WaylandSurface *surface)
   surface->is_opaque = TRUE;
 }
 
+void
+wayland_surface_fixate_size (WaylandSurface *surface)
+{
+  surface->fixed_size = TRUE;
+  xdg_toplevel_set_min_size (surface->xdg_toplevel,
+                             surface->default_width,
+                             surface->default_height);
+  xdg_toplevel_set_max_size (surface->xdg_toplevel,
+                             surface->default_width,
+                             surface->default_height);
+}
+
 const char *
 lookup_property_string (WaylandDisplay *display,
                         const char     *name)
