@@ -533,6 +533,7 @@ static struct gbm_bo *
 import_scanout_gbm_bo (MetaWaylandDmaBufBuffer  *dma_buf,
                        MetaGpuKms               *gpu_kms,
                        int                       n_planes,
+                       uint32_t                  drm_format,
                        gboolean                 *use_modifier,
                        GError                  **error)
 {
@@ -556,7 +557,7 @@ import_scanout_gbm_bo (MetaWaylandDmaBufBuffer  *dma_buf,
       import_with_modifier = (struct gbm_import_fd_modifier_data) {
         .width = dma_buf->width,
         .height = dma_buf->height,
-        .format = dma_buf->drm_format,
+        .format = drm_format,
         .num_fds = n_planes,
         .modifier = dma_buf->drm_modifier,
       };
@@ -583,7 +584,7 @@ import_scanout_gbm_bo (MetaWaylandDmaBufBuffer  *dma_buf,
       import_legacy = (struct gbm_import_fd_data) {
         .width = dma_buf->width,
         .height = dma_buf->height,
-        .format = dma_buf->drm_format,
+        .format = drm_format,
         .stride = dma_buf->strides[0],
         .fd = dma_buf->fds[0],
       };
@@ -605,14 +606,54 @@ import_scanout_gbm_bo (MetaWaylandDmaBufBuffer  *dma_buf,
 }
 #endif
 
+static gboolean
+has_modifier (GArray   *modifiers,
+              uint64_t  drm_modifier)
+{
+  int i;
+
+  for (i = 0; i < modifiers->len; i++)
+    {
+      if (drm_modifier == g_array_index (modifiers, uint64_t, i))
+        return TRUE;
+    }
+  return FALSE;
+}
+
+#ifdef HAVE_NATIVE_BACKEND
+static gboolean
+crtc_supports_modifier (MetaCrtcKms *crtc_kms,
+                        uint32_t     drm_format,
+                        uint64_t     drm_modifier)
+{
+  MetaKmsPlane *plane = meta_crtc_kms_get_assigned_primary_plane (crtc_kms);
+  GArray *crtc_modifiers;
+
+  g_return_val_if_fail (plane, FALSE);
+
+  crtc_modifiers = meta_kms_plane_get_modifiers_for_format (plane, drm_format);
+  if (!crtc_modifiers)
+    return FALSE;
+
+  if (drm_modifier == DRM_FORMAT_MOD_INVALID)
+    return TRUE;
+
+  return has_modifier (crtc_modifiers, drm_modifier);
+}
+#endif
+
 CoglScanout *
 meta_wayland_dma_buf_try_acquire_scanout (MetaWaylandBuffer     *buffer,
                                           CoglOnscreen          *onscreen,
+                                          ClutterStageView      *stage_view,
                                           const graphene_rect_t *src_rect,
                                           const MtkRectangle    *dst_rect)
 {
 #ifdef HAVE_NATIVE_BACKEND
   MetaWaylandDmaBufBuffer *dma_buf;
+  MetaRendererView *renderer_view = META_RENDERER_VIEW (stage_view);
+  MetaCrtc *crtc;
+  MetaCrtcKms *crtc_kms;
   MetaContext *context;
   MetaBackend *backend;
   MetaRenderer *renderer;
@@ -624,6 +665,8 @@ meta_wayland_dma_buf_try_acquire_scanout (MetaWaylandBuffer     *buffer,
   g_autoptr (CoglScanout) scanout = NULL;
   g_autoptr (GError) error = NULL;
   MetaDrmBufferFlags flags;
+  const MetaFormatInfo *format_info;
+  uint32_t drm_format;
   gboolean use_modifier;
   int n_planes;
 
@@ -631,6 +674,35 @@ meta_wayland_dma_buf_try_acquire_scanout (MetaWaylandBuffer     *buffer,
   if (!dma_buf)
     return NULL;
 
+  crtc = meta_renderer_view_get_crtc (renderer_view);
+  g_return_val_if_fail (META_IS_CRTC_KMS (crtc), NULL);
+  crtc_kms = META_CRTC_KMS (crtc);
+
+  format_info = meta_format_info_from_drm_format (dma_buf->drm_format);
+  g_assert (format_info);
+
+  if (format_info->opaque_substitute != DRM_FORMAT_INVALID &&
+      crtc_supports_modifier (crtc_kms,
+                              format_info->opaque_substitute,
+                              dma_buf->drm_modifier))
+    {
+      drm_format = format_info->opaque_substitute;
+    }
+  else if (crtc_supports_modifier (crtc_kms,
+                                   dma_buf->drm_format,
+                                   dma_buf->drm_modifier))
+    {
+      drm_format = dma_buf->drm_format;
+    }
+  else
+    {
+      meta_topic (META_DEBUG_RENDER,
+                  "DRM format 0x%x (0x%lx) not supported by primary plane",
+                  dma_buf->drm_format,
+                  dma_buf->drm_modifier);
+      return NULL;
+    }
+
   context = meta_wayland_compositor_get_context (dma_buf->manager->compositor);
   backend = meta_context_get_backend (context);
   renderer = meta_backend_get_renderer (backend);
@@ -644,7 +716,11 @@ meta_wayland_dma_buf_try_acquire_scanout (MetaWaylandBuffer     *buffer,
 
   device_file = meta_renderer_native_get_primary_device_file (renderer_native);
   gpu_kms = meta_renderer_native_get_primary_gpu (renderer_native);
-  gbm_bo = import_scanout_gbm_bo (dma_buf, gpu_kms, n_planes, &use_modifier,
+  gbm_bo = import_scanout_gbm_bo (dma_buf,
+                                  gpu_kms,
+                                  n_planes,
+                                  drm_format,
+                                  &use_modifier,
                                   &error);
   if (!gbm_bo)
     {
@@ -1297,37 +1373,6 @@ find_scanout_tranche_func (gconstpointer a,
     return -1;
 }
 
-static gboolean
-has_modifier (GArray   *modifiers,
-              uint64_t  drm_modifier)
-{
-  int i;
-
-  for (i = 0; i < modifiers->len; i++)
-    {
-      if (drm_modifier == g_array_index (modifiers, uint64_t, i))
-        return TRUE;
-    }
-  return FALSE;
-}
-
-static gboolean
-crtc_supports_modifier (MetaCrtcKms *crtc_kms,
-                        uint32_t     drm_format,
-                        uint64_t     drm_modifier)
-{
-  MetaKmsPlane *plane = meta_crtc_kms_get_assigned_primary_plane (crtc_kms);
-  GArray *crtc_modifiers;
-
-  g_return_val_if_fail (plane, FALSE);
-
-  crtc_modifiers = meta_kms_plane_get_modifiers_for_format (plane, drm_format);
-  if (!crtc_modifiers)
-    return FALSE;
-
-  return has_modifier (crtc_modifiers, drm_modifier);
-}
-
 static void
 ensure_scanout_tranche (MetaWaylandDmaBufSurfaceFeedback *surface_feedback,
                         MetaCrtc                         *crtc)
@@ -1375,10 +1420,24 @@ ensure_scanout_tranche (MetaWaylandDmaBufSurfaceFeedback *surface_feedback,
                            MetaWaylandDmaBufFormat,
                            i);
 
+          if (format.drm_modifier == DRM_FORMAT_MOD_INVALID)
+            continue;
+
           if (!crtc_supports_modifier (crtc_kms,
                                        format.drm_format,
                                        format.drm_modifier))
-            continue;
+            {
+              const MetaFormatInfo *format_info;
+
+              format_info = meta_format_info_from_drm_format (format.drm_format);
+              g_assert (format_info);
+
+              if (format_info->opaque_substitute == DRM_FORMAT_INVALID ||
+                  !crtc_supports_modifier (crtc_kms,
+                                           format_info->opaque_substitute,
+                                           format.drm_modifier))
+                continue;
+            }
 
           g_array_append_val (formats, format);
         }
