@@ -102,3 +102,10 @@ void meta_wayland_pointer_focus_surface (MetaWaylandPointer *pointer,
 MetaWaylandSurface * meta_wayland_pointer_get_implicit_grab_surface (MetaWaylandPointer *pointer);
 
 MetaWaylandPointerClient * meta_wayland_pointer_get_focus_client (MetaWaylandPointer *pointer);
+
+void meta_wayland_pointer_set_cursor_shape (MetaWaylandPointer *pointer,
+                                            MetaCursor          shape);
+
+gboolean meta_wayland_pointer_check_focus_serial (MetaWaylandPointer *pointer,
+                                                  struct wl_client   *client,
+                                                  uint32_t            serial);
