@@ -45,6 +45,8 @@ struct _MetaWaylandTransaction
 
   /* Sources for buffers which are not ready yet */
   GHashTable *buf_sources;
+
+  int64_t target_presentation_time_us;
 };
 
 struct _MetaWaylandTransactionEntry
@@ -60,6 +62,12 @@ struct _MetaWaylandTransactionEntry
   int y;
 };
 
+int64_t
+meta_wayland_transaction_get_target_presentation_time_us (const MetaWaylandTransaction *transaction)
+{
+  return transaction->target_presentation_time_us;
+}
+
 static MetaWaylandTransactionEntry *
 meta_wayland_transaction_get_entry (MetaWaylandTransaction *transaction,
                                     MetaWaylandSurface     *surface)
@@ -240,16 +248,34 @@ has_dependencies (MetaWaylandTransaction *transaction)
 {
   GHashTableIter iter;
   MetaWaylandSurface *surface;
+  MetaWaylandTransactionEntry *entry;
+
+  if (transaction->target_presentation_time_us)
+    return TRUE;
 
   if (transaction->buf_sources &&
       g_hash_table_size (transaction->buf_sources) > 0)
     return TRUE;
 
   g_hash_table_iter_init (&iter, transaction->entries);
-  while (g_hash_table_iter_next (&iter, (gpointer *) &surface, NULL))
+  while (g_hash_table_iter_next (&iter, (gpointer *) &surface,
+                                        (gpointer *) &entry))
     {
+      MetaSurfaceActor *actor;
+
       if (surface->transaction.first_committed != transaction)
         return TRUE;
+
+      if (!entry || !entry->state)
+        continue;
+
+      actor = meta_wayland_surface_get_actor (surface);
+      if (!actor || meta_surface_actor_is_effectively_obscured (actor) ||
+          !clutter_actor_is_mapped (CLUTTER_ACTOR (actor)))
+        continue;
+
+      if (entry->state->fifo_wait && surface->fifo_barrier)
+        return TRUE;
     }
 
   return FALSE;
@@ -283,6 +309,44 @@ meta_wayland_transaction_maybe_apply (MetaWaylandTransaction *transaction)
     }
 }
 
+gboolean
+meta_wayland_transaction_unblock_timed (MetaWaylandTransaction *transaction,
+                                        int64_t                 target_time_us)
+{
+  if (target_time_us < transaction->target_presentation_time_us)
+    return FALSE;
+
+  transaction->target_presentation_time_us = 0;
+
+  meta_wayland_transaction_maybe_apply (transaction);
+
+  return TRUE;
+}
+
+void
+meta_wayland_transaction_consider_surface (MetaWaylandSurface *surface)
+{
+  MetaWaylandTransaction *transaction;
+
+  transaction = surface->transaction.first_committed;
+  if (transaction)
+    meta_wayland_transaction_maybe_apply (transaction);
+}
+
+void
+meta_wayland_transaction_unblock_surface (MetaWaylandSurface *surface)
+{
+  if (!surface->fifo_barrier)
+    {
+      g_warning ("Attempting to unblock a surface with no fifo_barrier");
+      return;
+    }
+
+  surface->fifo_barrier = FALSE;
+
+  meta_wayland_transaction_consider_surface (surface);
+}
+
 static void
 meta_wayland_transaction_dma_buf_dispatch (MetaWaylandBuffer *buffer,
                                            gpointer           user_data)
@@ -389,6 +453,8 @@ meta_wayland_transaction_commit (MetaWaylandTransaction *transaction)
   g_autoptr (GPtrArray) placement_states = NULL;
   unsigned int num_placement_states = 0;
   int i;
+  gint64 max_time_us = 0;
+  MetaWaylandSurface *max_time_surface = NULL;
 
   g_hash_table_iter_init (&iter, transaction->entries);
   while (g_hash_table_iter_next (&iter,
@@ -413,6 +479,13 @@ meta_wayland_transaction_commit (MetaWaylandTransaction *transaction)
               g_ptr_array_add (placement_states, entry->state);
               num_placement_states++;
             }
+
+          if (entry->state->has_target_time &&
+              entry->state->target_time_us > max_time_us)
+            {
+              max_time_us = entry->state->target_time_us;
+              max_time_surface = surface;
+            }
         }
     }
 
@@ -425,6 +498,23 @@ meta_wayland_transaction_commit (MetaWaylandTransaction *transaction)
                                                        placement_state);
     }
 
+  /* If we have a time constraint, we always defer application until just before the
+   * appropriate frame clock tick.
+   */
+  if (max_time_us)
+    {
+      MetaSurfaceActor *actor = meta_wayland_surface_get_actor (max_time_surface);
+      ClutterFrameClock *frame_clock =
+        clutter_actor_pick_frame_clock (CLUTTER_ACTOR (actor), NULL);
+
+      if (frame_clock)
+        {
+          maybe_apply = FALSE;
+          transaction->target_presentation_time_us = max_time_us;
+          meta_wayland_compositor_add_timed_transaction (transaction->compositor, transaction);
+          clutter_frame_clock_add_future_time (frame_clock, max_time_us);
+        }
+    }
   transaction->committed_sequence = ++committed_sequence;
   transaction->node.data = transaction;
 
@@ -437,8 +527,6 @@ meta_wayland_transaction_commit (MetaWaylandTransaction *transaction)
     {
       if (surface->transaction.first_committed)
         {
-          MetaWaylandTransactionEntry *entry;
-
           entry = g_hash_table_lookup (surface->transaction.last_committed->entries,
                                        surface);
           entry->next_transaction = transaction;
