@@ -1616,6 +1616,49 @@ reload_window_opacity (MetaWindow    *window,
   meta_window_set_opacity (window, opacity);
 }
 
+static void
+reload_fullscreen_monitors (MetaWindow    *window,
+                            MetaPropValue *value,
+                            gboolean       initial)
+{
+  if (value->type != META_PROP_VALUE_INVALID)
+    {
+      if (value->v.cardinal_list.n_cardinals != 4)
+        {
+          meta_topic (META_DEBUG_X11,
+                      "_NET_WM_FULLSCREEN_MONITORS on %s has %d values instead of 4",
+                      window->desc, value->v.cardinal_list.n_cardinals);
+        }
+      else
+        {
+          MetaX11Display *x11_display = window->display->x11_display;
+          int top_xinerama_index, bottom_xinerama_index;
+          int left_xinerama_index, right_xinerama_index;
+          MetaLogicalMonitor *top, *bottom, *left, *right;
+
+          top_xinerama_index = (int)value->v.cardinal_list.cardinals[0];
+          bottom_xinerama_index = (int)value->v.cardinal_list.cardinals[1];
+          left_xinerama_index = (int)value->v.cardinal_list.cardinals[2];
+          right_xinerama_index = (int)value->v.cardinal_list.cardinals[3];
+
+          top =
+            meta_x11_display_xinerama_index_to_logical_monitor (x11_display,
+                                                                top_xinerama_index);
+          bottom =
+            meta_x11_display_xinerama_index_to_logical_monitor (x11_display,
+                                                                bottom_xinerama_index);
+          left =
+            meta_x11_display_xinerama_index_to_logical_monitor (x11_display,
+                                                                left_xinerama_index);
+          right =
+            meta_x11_display_xinerama_index_to_logical_monitor (x11_display,
+                                                                right_xinerama_index);
+
+          meta_window_update_fullscreen_monitors (window, top, bottom, left, right);
+        }
+    }
+}
+
 #define RELOAD_STRING(var_name, propname) \
   static void                                       \
   reload_ ## var_name (MetaWindow    *window,       \
@@ -1709,6 +1752,7 @@ meta_x11_display_init_window_prop_hooks (MetaX11Display *x11_display)
     { x11_display->atom__NET_WM_STRUT_PARTIAL, META_PROP_VALUE_INVALID, reload_struts, NONE },
     { x11_display->atom__NET_WM_BYPASS_COMPOSITOR, META_PROP_VALUE_CARDINAL,  reload_bypass_compositor, LOAD_INIT | INCLUDE_OR },
     { x11_display->atom__NET_WM_WINDOW_OPACITY, META_PROP_VALUE_CARDINAL, reload_window_opacity, LOAD_INIT | INCLUDE_OR },
+    { x11_display->atom__NET_WM_FULLSCREEN_MONITORS, META_PROP_VALUE_CARDINAL_LIST, reload_fullscreen_monitors, LOAD_INIT | INIT_ONLY },
     { 0 },
   };
   MetaWindowPropHooks *table;
