@@ -28,6 +28,7 @@
 #include "meta/compositor.h"
 #include "meta/window.h"
 #include "x11/meta-sync-counter.h"
+#include "x11/meta-x11-frame.h"
 
 G_BEGIN_DECLS
 
@@ -100,6 +101,9 @@ void meta_window_x11_surface_rect_to_client_rect (MetaWindow   *window,
 
 MtkRectangle meta_window_x11_get_client_rect    (MetaWindowX11 *window_x11);
 
+META_EXPORT_TEST
+MetaFrame * meta_window_x11_get_frame (MetaWindow *window);
+
 gboolean meta_window_x11_can_unredirect          (MetaWindowX11 *window_x11);
 
 MetaSyncCounter * meta_window_x11_get_sync_counter (MetaWindow *window);
@@ -112,3 +116,5 @@ gboolean meta_window_x11_has_alpha_channel (MetaWindow *window);
 
 META_EXPORT
 Window meta_window_x11_get_xwindow (MetaWindow *window);
+
+void meta_window_x11_configure (MetaWindow *window);
